import sys
import traceback
import configparser
import json
from tempfile import NamedTemporaryFile
from datetime import datetime

from uzemszunet.utils import send_email, format_email, get_config_path

import requests
import pandas


cfg = configparser.ConfigParser()
cfg.read(get_config_path())


TELEPULESEK = json.loads(cfg.get('Uzemszunet', 'telepulesek'))
NOTIFICATION_DAYS = json.loads(cfg.get('Uzemszunet', 'notifcation_days'))


def dl_eon_file():
    """
    Letölti az EON üzemszünet fájlt.
    """
    dl_url = cfg.get('EON', 'xls_url')
    r = requests.get(dl_url, stream=True)
    r.raise_for_status()

    f = NamedTemporaryFile(mode="wb+")
    f.write(r.content)
    return f


def parse_eon_file(f, aramszunetek):
    """
    Analizálja a fájlt és az összes konfigurációban megadott
    településre lekérdezi az tervezett üzemszüneteket!
    """
    xls = pandas.read_excel(f, sheet_name="Áram", header=1)
    xls_dict = xls.to_dict()

    telepulesek = xls_dict["Település"]
    now = datetime.now().date()

    for index, telepules in enumerate(telepulesek.items()):
        if telepules[1] in TELEPULESEK:
            datum = xls_dict["Dátum"][index]
            dt = datetime.strptime(datum[0:10], "%Y-%m-%d").date()
            diff = (dt - now).days

            if diff not in NOTIFICATION_DAYS:
                continue

            if aramszunetek.get(datum) is None:
                aramszunetek[datum] = {}
            if aramszunetek.get(datum).get(telepules[1]) is None:
                aramszunetek[datum][telepules[1]] = []

            aramszunetek[datum][telepules[1]].append(
                {
                    "utca": xls_dict["Utca"][index],
                    "terulet": xls_dict["Terület"][index],
                    "hazszam_tol": xls_dict["Házszám(tól)"][index],
                    "hazszam_ig": xls_dict["Házszám(ig)"][index],
                    "idopont_tol": xls_dict["Időpont(tól)"][index],
                    "idopont_ig": xls_dict["Időpont(ig)"][index],
                    "megjegyzes": xls_dict["Megjegyzés"][index],
                    "szolgaltato": "EON"
                }
            )
    return aramszunetek


def lekerdez(email=False):
    try:
        eon_file = dl_eon_file()
        uzemszunetek = {}

        parse_eon_file(eon_file, uzemszunetek)

        if len(uzemszunetek) > 0 and email:
            html = format_email(uzemszunetek)
            send_email(
                html,
                cfg.get('Email', 'smtp_host'),
                cfg.get('Email', 'user'),
                cfg.get('Email', 'password'),
                cfg.get('Email', 'to_mail'),
                'Tervezett üzemszünetek'
            )
        return uzemszunetek
    except Exception as e:
        exc_type, exc_value, exc_tb = sys.exc_info()
        exc_text = traceback.format_exception(exc_type, exc_value, exc_tb)

        if email:
            send_email(
                'Hiba történt:' + str(exc_text),
                cfg.get('Email', 'smtp_host'),
                cfg.get('Email', 'user'),
                cfg.get('Email', 'password'),
                cfg.get('Email', 'to_mail'),
                'Tervezett üzemszünetek HIBA'
            )
        print(exc_text)



import smtplib
import ssl
import sys
import os
from pathlib import Path
from datetime import datetime
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText


def get_config_path():
    platform = sys.platform
    if platform == 'win32':
        return os.path.join(
            os.getenv('APPDATA'), 'uzemszunet', 'uzemszunet.cfg'
        )
    else:
        # Unix alapú rendszereken talán ezt meg fogja enni (Linuxon tuti megy)
        # TODO ^ Kivizsgálni
        return os.path.join(
            Path.home(), '.config', 'uzemszunet', 'uzemszunet.cfg'
        )


def format_email(uzemszunetek):
    """
    Megformázza az E-mail-t küldés előtt.
    """
    email_html = "Várható üzemszünetek:<br />"

    for datum in uzemszunetek.items():
        email_html += "<h3>{}</h3>".format(datum[0][0:10])
        for telepules in datum[1].items():
            email_html += "<h4>{}</h4>".format(telepules[0])

            email_html += "<ul>"
            for helyszin in telepules[1]:
                email_html += "<li><b>{0};</b> {1}-{2}; {3}-{4}; {5}</li>".format(
                    helyszin.get("utca"),
                    helyszin.get("hazszam_tol"),
                    helyszin.get("hazszam_ig"),
                    helyszin.get("idopont_tol"),
                    helyszin.get("idopont_ig"),
                    helyszin.get("megjegyzes")
                )
            email_html += "</ul> <br />"
        return email_html


def send_email(
    text,
    smtp_host,
    user, password,
    to_mail,
    subject,
    smtp_port=465
):
    """
    Email elküldése.
    """
    try:
        message = MIMEMultipart("alternative")

        part1 = MIMEText(text, "plain")
        part2 = MIMEText(text, "html")

        message.attach(part1)
        message.attach(part2)

        message["Subject"] = subject
        message["From"] = user
        message["TO"] = to_mail

        context = ssl.create_default_context()
        with smtplib.SMTP_SSL(smtp_host, smtp_port, context=context) as server:
            server.login(user, password)
            server.sendmail(
                user, to_mail, message.as_string()
            )
    except Exception as e:
        print(e)



import os
import sys
import shutil
import argparse
from pprint import pprint
from uzemszunet.utils import get_config_path


def copy_config(path):
    os.makedirs(os.path.dirname(path), exist_ok=True)
    package_dir = os.path.dirname(os.path.abspath(__file__))
    shutil.copy(os.path.join(package_dir, 'uzemszunet.cfg'), path)


def main():
    cfg_path = get_config_path()
    if not os.path.isfile(cfg_path):
        copy_config(cfg_path)
        sys.exit(
            f'Konfigurációs fájl létrehozva. Állítsd  be az E-mail címed: {cfg_path}'
        )

    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--email',
        help='E-mail-ben ki lesz küldve az eredmény.',
        action='store_true'
    )
    args = parser.parse_args()

    from uzemszunet.uzemszunet import lekerdez
    res = lekerdez(args.email)

    if not args.email:
        pprint(res)


if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="uzemszunet",
    version="0.0.2",
    author="Ferenc Nánási",
    author_email="husudosu94@gmail.com",
    description="Üzemszünetek lekérdezése",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/husudosu/uzemszunet",
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    install_requires=[
            "requests",
            "pandas",
            "xlrd"
    ],
    entry_points={
        'console_scripts': [
            'uzemszunet = uzemszunet.__main__:main'
        ]
    },
    package_data={'uzemszunet': ['uzemszunet.cfg']},
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'License :: Public Domain',
        'Intended Audience :: System Administrators',
        'Natural Language :: Hungarian',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Communications :: Email',
        'Topic :: Utilities',
    ]
)
